import openai
import json
import numpy as np
from sklearn.metrics.pairwise import cosine_similarity

openai.api_key = 'OPEN-API-KEY'

# Load embeddings and chunks for the query system
print("Loading embeddings and chunks from file...")
with open('embeddings.json', 'r') as f:
    data = json.load(f)

stored_embeddings = np.array(data['embeddings'])
stored_chunks = data['chunks']
print("Loaded embeddings and chunks.")

# Function to get embedding for a query
def get_query_embedding(query):
    response = openai.Embedding.create(
        model="text-embedding-ada-002",
        input=query
    )
    return response['data'][0]['embedding']

# Function to find the most similar text chunks
def find_most_similar_chunks(query):
    query_embedding = get_query_embedding(query)
    similarities = cosine_similarity([query_embedding], stored_embeddings)
    most_similar_indices = np.argsort(similarities[0])[::-1]
    return [stored_chunks[idx] for idx in most_similar_indices[:5]]

# Function to create a response based on the most similar chunks
def create_response(query):
    relevant_info = find_most_similar_chunks(query)
    combined_info = "\n\n".join(relevant_info)
    print("Most similar chunks found:", combined_info)
    response = openai.ChatCompletion.create(
        model="gpt-4",
        messages=[
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": f"Using the following information from our documentation, answer the query:\n\nDocumentation: {combined_info}\n\nQuery: {query}"}
        ],
        max_tokens=300
    )
    return response.choices[0].message['content']

print("You can start asking questions now. Type 'exit' to stop.")

# Interactive loop for user queries
while True:
    user_query = input("Ask your question: ")
    if user_query.lower() == 'exit':
        break
    print("Processing query...")
    assistant_response = create_response(user_query)
    print("Assistant's response:")
    print(assistant_response)